
DROP TABLE IF EXISTS Person CASCADE;
DROP TABLE IF EXISTS Posting CASCADE;
DROP TABLE IF EXISTS Comment CASCADE;
DROP TABLE IF EXISTS Information CASCADE;
DROP TABLE IF EXISTS Data CASCADE;
DROP TABLE IF EXISTS Text CASCADE;
DROP TABLE IF EXISTS Follows CASCADE;
DROP TABLE IF EXISTS Tags CASCADE;
DROP TABLE IF EXISTS Likes CASCADE;
DROP TABLE IF EXISTS Retweet CASCADE;
DROP TABLE IF EXISTS Comment CASCADE;

CREATE TABLE Person (
    Nickname      TEXT,
    Firstname     TEXT,
    Lastname      TEXT,
    Password      TEXT NOT NULL,
    Birthdate     TIMESTAMP,
    CONSTRAINT person_pk PRIMARY KEY (Nickname),
    CONSTRAINT password_length_check CHECK (char_length(Password)>0)
);

CREATE TABLE Posting (
    Nickname  TEXT,
    PostingID BIGINT,
    Time      TIMESTAMP NOT NULL,
    location  TEXT,
    CONSTRAINT posting_person_fk FOREIGN KEY(Nickname) REFERENCES Person(Nickname),
    CONSTRAINT posting_pk PRIMARY KEY(Nickname, PostingID)
);

CREATE TABLE Comment (
    CommentID         BIGINT,
    PostPostingID     BIGINT,
    PostNickname      TEXT,
    CommenterNickname TEXT,
    CommentText       TEXT,
    CONSTRAINT comment_posting_fk FOREIGN KEY(PostNickname, PostPostingID) REFERENCES Posting(Nickname, PostingID),
    CONSTRAINT comment_person_fk FOREIGN KEY(CommenterNickname) REFERENCES Person(Nickname),
    CONSTRAINT comment_pk PRIMARY KEY(CommentID, PostPostingID, PostNickname)
);

CREATE TABLE Information (
    InformationID BIGINT,
    Nickname      TEXT,
    PostingID     BIGINT,
    size          BIGINT,
    CONSTRAINT information_posting_fk FOREIGN KEY(Nickname, PostingID) REFERENCES Posting(Nickname, PostingID),
    CONSTRAINT information_pk PRIMARY KEY(InformationID)
);

CREATE TABLE Data (
    InformationID BIGINT,
    Link          TEXT NOT NULL,
    CONSTRAINT data_information_fk FOREIGN KEY(InformationID) REFERENCES Information(InformationID),
    CONSTRAINT data_pk PRIMARY KEY(InformationID),
    CONSTRAINT link_length_check CHECK (char_length(Link)>0)
);

CREATE TABLE Text (
    InformationID BIGINT,
    Message       TEXT NOT NULL,
    CONSTRAINT text_information_fk FOREIGN KEY(InformationID) REFERENCES Information(InformationID),
    CONSTRAINT text_pk PRIMARY KEY(InformationID),
    CONSTRAINT message_length_check CHECK (char_length(Message)>0)
);

CREATE TABLE Follows (
    FollowerNickname TEXT,
    FolloweeNickname TEXT,
    CONSTRAINT follows_person_follower_fk FOREIGN KEY(FollowerNickname) REFERENCES Person(Nickname),
    CONSTRAINT follows_person_followee_fk FOREIGN KEY(FolloweeNickname) REFERENCES Person(Nickname),
    CONSTRAINT follows_pk PRIMARY KEY(FollowerNickname, FolloweeNickname)
);

CREATE TABLE Likes (
    Nickname TEXT,
    PostingID BIGINT,
    LikeeNickname TEXT,
    FOREIGN KEY(Nickname, PostingID) REFERENCES Posting(Nickname, PostingID),
    FOREIGN KEY(LikeeNickname) REFERENCES Person(Nickname),
    PRIMARY KEY(Nickname, PostingID, LikeeNickname)
);

CREATE TABLE Retweet(
    Nickname TEXT,
    PostingID BIGINT,
    RetweetNickname Text,
    FOREIGN KEY(Nickname, PostingID) REFERENCES Posting(Nickname, PostingID),
    FOREIGN KEY(RetweetNickname) REFERENCES Person(Nickname),
    PRIMARY KEY(Nickname, PostingID, RetweetNickname)
);

CREATE TABLE Tags (
    Nickname  TEXT,
    PostingID BIGINT,
    Tag       TEXT NOT NULL,
    CONSTRAINT tag_posting_fk FOREIGN KEY(Nickname, PostingID) REFERENCES Posting(Nickname, PostingID),
    CONSTRAINT tag_pk PRIMARY KEY(Nickname, PostingID, Tag),
    CONSTRAINT tag_length_check CHECK (char_length(Tag)>0)
);
